#include <stdlib.h>
#include <time.h>

#define tollerance 0
#define max_value  32000
#define win_value (max_value - 2)

// Boolean values.

#define False 0
#define True  1

// Color values.

#define Player1 0
#define Player2 1

#define other_player(sid) ((sid == Player1) ? Player2 : Player1)
#define abs_v(val) (((val) < 0) ? -(val) : (val))


// Class to hold a move.

class Gen_move_type {

public :

    int            move;
    int            val;
    Gen_move_type *next;
    Gen_move_type *continuation;
    Gen_move_type *next_lev_moves;

    Gen_move_type() {}

    Gen_move_type(int vl, Gen_move_type *nxt)
        { val = vl; next = nxt; }

};


class Position_type {

public :

    //
    //  Data.
    //

    char    mover;
    int    scores[2];
    int     last_moves[2];
    int     hoops_incor[2];
    int     hoop_jump_needed[2];
    int     back_to_hoop[2];
    int     current_move;
    Position_type  *next;

    //
    //  Member functions.
    //

    Gen_move_type *gen_moves();
    void make_move_on_board(Gen_move_type *move);
    void update_move_list(int *done);
    void setup_board();
    int evaluate(char comp_side);

private :

    void append_move_list();
};


static Gen_move_type *glob__move_free_list = NULL;
static Position_type  glob__current_position1;
static Position_type  glob__current_position2;
static int            glob__depth_reached;
static int            glob__max_depth[2];
static int            glob__max_depth_reached;
static long           glob__posits_evaled;
static int            glob__alpha_betas[50];
static long           glob__elapsed_time;
static long           glob__end_time;
static long           glob__start_time;


// Routines to handle creating and deleting positions.

static Position_type *new_position();
static void delete_position(Position_type *posit);


//
// Move free list functions.
//

static Gen_move_type *new_move_type();
static Gen_move_type *new_move_type(int vl, Gen_move_type *nxt);
static void delete_move_type(Gen_move_type *move);


// Declare functions.

int timeleft();

static void initialize_variables(Position_type *posit);
static int generate_computer_move(Position_type *posit);
static int give_me_a_value(Position_type *posit, Gen_move_type *move, int depth, int curr_depth, int alpha_beta, char comp_side);



int move1(int score1, int score2, int last_move)
   {
      Gen_move_type prev_move;
      int           done;
      int           chosen_move;

      if(score1 == 0 && score2 == 0) {
          initialize_variables(&glob__current_position1);
      }

      if(score2 != 0) {
         if(score2 < glob__current_position1.scores[Player2]) {
             prev_move.move = -last_move;
         } else {
             prev_move.move = last_move;
         }
         glob__current_position1.make_move_on_board(&prev_move);
         glob__current_position1.update_move_list(&done);
      }
      chosen_move = generate_computer_move(&glob__current_position1);
      return(chosen_move);
   }


int move2(int score1, int score2, int last_move)
   {
      Gen_move_type prev_move;
      int           done;
      int           chosen_move;

      if(score1 == 0 && score2 == 0) {
          initialize_variables(&glob__current_position2);
      }

      if(score1 != 0) {
         if(score1 < glob__current_position2.scores[Player1]) {
             prev_move.move = -last_move;
         } else {
             prev_move.move = last_move;
         }
         glob__current_position2.make_move_on_board(&prev_move);
         glob__current_position2.update_move_list(&done);
      }

      chosen_move = generate_computer_move(&glob__current_position2);
      return(chosen_move);
   }


static int check_opening_library(Position_type  *posit)
    {
        if(posit->current_move == 1) {
            if(posit->mover == Player1) {
               return(7);
            } else {
               switch (posit->last_moves[Player1]) {
                   case 1 : return(4);
                   case 2 : return(4);
                   case 3 : return(4);
                   case 4 : return(3);
                   case 5 : return(1);
                   case 6 : return(7);
                   case 7 : return(6);
                   case 8 : return(5);
                   default : return(99);
               }
            }
        }
        return(99);
    }


static Gen_move_type *sort_moves(
        Gen_move_type *move_top   // Points to the first move in list.
    )
    // This procedure will sort the list according to the variable Value.
    // Higher values will come first.  An insertion sort is used.  This
    // Could clearly be sped up.
    {
        Gen_move_type *end_move;
        Gen_move_type *temp;
        Gen_move_type *temp_org;
        Gen_move_type *temp1;


        if(move_top) {
            temp_org = move_top->next;
            move_top->next = NULL;
            end_move = move_top;
            while(temp_org) {
                if(end_move->val >= temp_org->val) {

                    // Put this at the end of the list.

                    end_move->next = temp_org;
                    end_move = temp_org;
                    temp_org = temp_org->next;
                    end_move->next = NULL;
                } else if(move_top->val < temp_org->val) {

                    // Insert this in front of move_top.

                    temp = move_top;
                    move_top = temp_org;
                    temp_org = temp_org->next;
                    move_top->next = temp;
                } else {

                    // Insert at the proper point in the list.

                    temp = move_top;
                    while(temp->next) {
                        if(temp->next->val < temp_org->val) break;
                        else temp = temp->next;
                    }
                    temp1 = temp_org;
                    temp_org = temp1->next;
                    temp1->next = temp->next;
                    if(!temp->next) end_move = temp1;
                    temp->next = temp1;
                }
            }
        }
        return(move_top);
    }


static Gen_move_type *sort_moves_reverse(
        Gen_move_type *move_top   // Points to the first move in list.
    )
    // This procedure will sort the list according to the variable Value.
    // Lower values will come first.  An insertion sort is used.  This
    // Could clearly be sped up.
    {
        Gen_move_type *end_move;
        Gen_move_type *temp;
        Gen_move_type *temp_org;
        Gen_move_type *temp1;


        if(move_top) {
            temp_org = move_top->next;
            move_top->next = NULL;
            end_move = move_top;
            while(temp_org) {
                if(end_move->val <= temp_org->val) {

                    // Put this at the end of the list.

                    end_move->next = temp_org;
                    end_move = temp_org;
                    temp_org = temp_org->next;
                    end_move->next = NULL;
                } else if(move_top->val > temp_org->val) {

                    // Insert this in front of move_top.

                    temp = move_top;
                    move_top = temp_org;
                    temp_org = temp_org->next;
                    move_top->next = temp;
                } else {

                    // Insert at the proper point in the list.

                    temp = move_top;
                    while(temp->next) {
                        if(temp->next->val > temp_org->val) break;
                        else temp = temp->next;
                    }
                    temp1 = temp_org;
                    temp_org = temp1->next;
                    temp1->next = temp->next;
                    if(!temp->next) end_move = temp1;
                    temp->next = temp1;
                }
            }
        }
        return(move_top);
    }


static void free_list(
        Gen_move_type *point
    )
    // This function will dispose of all of the
    // nodes from point down in the list.
    {
        Gen_move_type *temp;

        while(point) {
            temp  = point;
            point = point->next;
            delete_move_type(temp);
        }
    }


static int generate_computer_move(Position_type *posit)
    // This function will generate a move for the given side and
    // the given position.  It does this by looking ahead to the
    // specified depth in the game tree starting at the given
    // position.  It uses a modified Alpha-Beta search method.
    {
        char            computer_side;
        int             curr_level;
        int             dn;
        int            *done;
        int             good_move_count;
        int             i;
        int             leave_loop;
        int             level;
        int             lib_move;
        long            max_time;
        long            max_time_90_percent;
        int             max_val;
        int             min_max;
        Gen_move_type  *move_top;
        int             picked_move;
        int             ratio;
        int             remaining_time;
        int             ret_val;
        Gen_move_type  *temp_move;
        Gen_move_type  *the_move;
        int             value;


        level = 50;
        done = &dn;
        glob__posits_evaled = 0;
        glob__depth_reached = 0;
        leave_loop = False;

        lib_move = check_opening_library(posit);
        if(lib_move != 99) {

            the_move = new_move_type();
            the_move->move = lib_move;
            the_move->next = NULL;

            // Make move on board and other things.
    
            posit->make_move_on_board(the_move);
            posit->update_move_list(done);
            ret_val = the_move->move;
            delete_move_type(the_move);
            return(ret_val);
         }

        // We set the max time that we should try to use for this move to
        // (TL - (TL/(2*ML))) / ML
        // which always leaves half a move's worth of time for overshoot.

        remaining_time = timeleft();
        if(remaining_time == 0) remaining_time = 1;
        if(remaining_time < 5) {
            max_time = 1;
        } else {
            if(posit->current_move < 5) {
                max_time = 4;
            } else {
                if(posit->scores[posit->mover] > 70) {
                    ratio = ((108 - posit->scores[posit->mover]) / remaining_time) - 1;
                } else {
                    ratio = (108 - posit->scores[posit->mover]) / remaining_time;
                }
                if(ratio >= 3) {
                    max_time = 2;
                } else if(ratio == 2) {
                    max_time = 2;
                } else if(ratio <= 0) {
                    max_time = 4;
                } else {
                    max_time = 3;
                }
            }
        }
        max_time_90_percent = max_time; // 9 * max_time / 10;
        time(&glob__start_time);

        computer_side = posit->mover;
        move_top = posit->gen_moves();
        if(move_top->next == NULL) {
            the_move = move_top;
            the_move->continuation = NULL;
        } else {
            glob__max_depth_reached = 0;
            for(curr_level = 1; curr_level <= level && curr_level - 2 <= glob__max_depth_reached; curr_level++) {

                glob__max_depth_reached = 0;
                glob__depth_reached = curr_level;
                min_max = -max_value + 1;
                glob__alpha_betas[1] = max_value - 1;
                glob__alpha_betas[2] = min_max;
                for(temp_move = move_top; temp_move; temp_move = temp_move->next) {
                    value = give_me_a_value(posit, temp_move, curr_level, 1, min_max, computer_side);
                    temp_move->val = value;
                    if(value == win_value) {
                        leave_loop = True;
                        break;
                    }
                    if(-max_value + tollerance <= value) {
                        if(value - tollerance > min_max) {
                            min_max = value - tollerance;
                            glob__alpha_betas[2] = min_max;
                        }
                    } else {
                        min_max = -max_value;
                        glob__alpha_betas[2] = min_max;
                    }

                    // Make sure we haven't exceeded our time limit, but
                    // that we have at least completed level 1.

                    time(&glob__end_time);
                    glob__elapsed_time = glob__end_time - glob__start_time;
                    if(glob__elapsed_time >= max_time_90_percent && curr_level > 1) {
                        leave_loop = True;

                        // Only look at moves looked at on this level.

                        the_move = temp_move->next;
                        temp_move->next = NULL;

                        // Free move lists for second level.

                        temp_move = the_move;
                        while(temp_move) {
                            free_list(temp_move->next_lev_moves);
                            temp_move = temp_move->next;
                        }
                        free_list(the_move);
                        glob__depth_reached = -curr_level;

                        break;
                    }
                }

                move_top = sort_moves(move_top);

                // We have found a win, so we can exit.

                if(move_top->val >= win_value - (tollerance + 1) * 32) leave_loop = True;
                if(move_top->val <= -win_value + (tollerance + 1) * 32) leave_loop = True;

                // If there is only one move that doesn't lose then exit the loop.

                if(move_top->next && move_top->next->val == -win_value) leave_loop = True;

                // If we wouldn't gain anything by looking another level deeper
                // then save the time for use later.  It seems to take about
                // the same amount of time to analyse the first move of level
                // n+1 as it did to analyse the entire level n.  If we can't
                // look at at least the first two moves in the next level then
                // don't bother.

                if(glob__elapsed_time + glob__elapsed_time + glob__elapsed_time > max_time) {
                    if(glob__depth_reached >= 0) glob__depth_reached = curr_level;
                    break;
                }

                if(leave_loop) {
                    break;
                }
            }

            // Here we choose a random move of those with values within tollerance.

            max_val = move_top->val;
            temp_move = move_top;
            good_move_count = 0;
            while(temp_move && temp_move->val >= max_val - tollerance) {
                good_move_count++;
                temp_move = temp_move->next;
            }
            picked_move = rand() % good_move_count;
            for(i = 1, the_move = move_top; i < picked_move; i++) the_move = the_move->next;
        }
        temp_move = the_move;
        the_move = new_move_type();
        *the_move = *temp_move;

        // Free lists of moves on second level before freeing this list.

        temp_move = move_top;
        while(temp_move) {
            free_list(temp_move->next_lev_moves);
            temp_move = temp_move->next;
        }

        free_list(move_top);

        // Make move on board and other things.

        posit->make_move_on_board(the_move);
        posit->update_move_list(done);
        ret_val = the_move->move;
        delete_move_type(the_move);
        return(ret_val);
    }

static int give_me_a_value(
        Position_type  *old_posit,
        Gen_move_type  *move,
        int             depth,
        int             curr_depth,
        int             alpha_beta,
        char            comp_side
    )
    {
        Gen_move_type *last_evaluated;
        int            min_max;
        Gen_move_type *move_top;
        Position_type *posit;
        int            ret_val;
        Gen_move_type *temp;
        Gen_move_type *temp1;
        int            value;


        // Copy the position.

        posit = new_position();
        *posit = *old_posit;

        // Make the move on the local board.

        posit->make_move_on_board(move);

        // Return the evaluation of this position if we are at maximum depth.

        if(curr_depth == depth) {
            value = posit->evaluate(comp_side);

            if(value == -win_value) value = -win_value + depth;
            if(value == win_value) value = win_value - depth;
            delete_position(posit);
            glob__max_depth_reached = curr_depth;
            return(value);
        }

        // Generate all possible moves from this position for the side to move.
        // Check for game over.

        if(curr_depth == 1 && depth > 2) {
            move_top = move->next_lev_moves;
        } else {
            move_top = posit->gen_moves();
        }

        if(move_top) {
            temp  = move_top;
            temp1 = temp;
            min_max = glob__alpha_betas[curr_depth];

            // Set up values for processing based on color.

            if(posit->mover == comp_side) {
                glob__alpha_betas[curr_depth + 2] = -max_value + 1;
                value   = -max_value;

                while(temp && value <= alpha_beta) {
                    value = give_me_a_value(posit, temp, depth, curr_depth + 1, min_max, comp_side);
                    temp->val = value;
                    if(value - tollerance > min_max) {
                        min_max = value - tollerance;
                        glob__alpha_betas[curr_depth + 2] = min_max;
                    } else if(value - tollerance > glob__alpha_betas[curr_depth + 2]) {
                        glob__alpha_betas[curr_depth + 2] = value - tollerance;
                    }
                    if(temp1->val <= temp->val) {
                        temp1 = temp;
                    }
                    last_evaluated = temp;
                    temp = temp->next;
                }
            } else {
                glob__alpha_betas[curr_depth + 2] = max_value - 1;
                value   = max_value;

                while(temp && value >= alpha_beta) {
                    value = give_me_a_value(posit, temp, depth, curr_depth + 1, min_max, comp_side);
                    temp->val = value;
                    if(value + tollerance < min_max) {
                        min_max = value + tollerance;
                        glob__alpha_betas[curr_depth + 2] = min_max;
                    } else if(value + tollerance < glob__alpha_betas[curr_depth + 2]) {
                        glob__alpha_betas[curr_depth + 2] = value + tollerance;
                    }
                    if(temp1->val >= temp->val) {
                        temp1 = temp;
                    }
                    last_evaluated = temp;
                    temp = temp->next;
                }
            }

            ret_val = temp1->val;

            if(curr_depth == 1) {

                // Sort the level 2 moves as far as we analysed.
                // First cut the list where we stopped evaluating.
                // Sort the first part of the list, then append the
                // saved portion of the list to the end of the sorted list.

                temp = last_evaluated->next;
                last_evaluated->next = NULL;
                move_top = sort_moves_reverse(move_top);
                while(last_evaluated->next) last_evaluated = last_evaluated->next;
                last_evaluated->next = temp;
                move->next_lev_moves = move_top;
            } else {
                free_list(move_top);
            }

            delete_position(posit);

            return(ret_val);

        } else {

            // Here the game is over.

            if(posit->back_to_hoop[posit->mover]) {
                if(comp_side == posit->mover) {
                   value = -win_value;
                } else {
                   value = win_value;
                }
            } else {
                value = posit->evaluate(comp_side);
            }

            if(curr_depth > glob__max_depth_reached) {
                glob__max_depth_reached = curr_depth;
            }
            delete_position(posit);

            return(value);
        }
    }


// Static values used for to speed up various operations.

static int count_back_to_hoop[110] =
   { 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,    // For 0.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 10.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 20.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 30.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 40.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 50.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 60.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 70.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 80.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,    // For 90.
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9 };  // For 100.


static Gen_move_type *moves_all;
static Gen_move_type *moves_priority_1;
static Gen_move_type *moves_priority_2;
static Gen_move_type *moves_priority_3;

// Points to free list of positions.

static Position_type *pos_free_list = NULL;



static void initialize_variables(Position_type *posit)
    // This function will initialize all of the variables, pointers, and
    // conditions needed for the program to play.
    {
        time_t   current_time;


        posit->setup_board();

        glob__max_depth[Player1]        = 50;
        glob__max_depth[Player2]        = 50;

        time(&current_time);
        srand((unsigned int)current_time);
    }


void Position_type::update_move_list(
        int            *done
    )
    // It will record the move in the move file.
    {
        // Check to see if this is the end of the game.

        if(mover == Player1) {
            if((scores[Player1] > 100 && !back_to_hoop[Player1]) ||
               (scores[Player2] > 100 && !back_to_hoop[Player2])) {
                *done = True;
            }
        }
        if(hoops_incor[Player1] >= 2) {
            *done = True;
        }
        if(hoops_incor[Player2] >= 2) {
            *done = True;
        }
    }


static Position_type *new_position()
    // Returns a pointer to a new position.
    // A free list is used to minimize time spent in malloc.
    {
        Position_type *new_posit;


        if(pos_free_list) {
            new_posit = pos_free_list;
            pos_free_list = new_posit->next;
        } else {
            new_posit = new Position_type;
        }
        return(new_posit);
    }


static void delete_position(Position_type *posit)
    // Puts this position on the free list for later use.
    {
        posit->next = pos_free_list;
        pos_free_list = posit;
    }


#define link_move_into_list(move_list, move) { move->next = move_list; move_list = move; }

Gen_move_type *Position_type::gen_moves()
    // This function will generate all legal moves from this position
    // and return a list containing them.  The list will be ordered
    // generally by priority.  See the comments to add_move and
    // append_move_list for more on how this is done.
    {
        int            i;
        Gen_move_type *new_move;
        char           other_side;
        int            none_found;
        int            temp_int;
        int            abs_last_move;
        int            other_side_back;


        moves_priority_1 = NULL;
        moves_priority_2 = NULL;
        moves_priority_3 = NULL;

        other_side = other_player(mover);
        abs_last_move = abs_v(last_moves[other_side]);
        other_side_back = count_back_to_hoop[scores[other_side]];

        // See if the game is already over.

        if(hoops_incor[mover] >= 2 || hoops_incor[other_side] >= 2) {
            return(NULL);
        }
        if(mover == Player1 && ((scores[Player1] > 100 && !back_to_hoop[Player1]) ||
                                (scores[Player2] > 100 && !back_to_hoop[Player2]))) {
            return(NULL);
        }

        if(back_to_hoop[mover]) {
            for(i = count_back_to_hoop[scores[mover]] + 1; i <= 8; i++) {
                if(other_side_back == 0 ||
                   (i != abs_last_move &&
                    i != 9 - abs_last_move)) {
                    new_move = new_move_type();
                    new_move->move = -i;
                    link_move_into_list(moves_priority_1, new_move);
                }
            }
        } else if(count_back_to_hoop[scores[mover]] == 0) {

            // We may have to pass here if opponent used the move we need.

            new_move = new_move_type();
            if(other_side_back == 0 ||
               (abs_last_move != hoop_jump_needed[mover] &&
                9 - abs_last_move != hoop_jump_needed[mover])) {
                new_move->move = hoop_jump_needed[mover];
            } else {
                new_move->move = 0;
            }
            link_move_into_list(moves_priority_1, new_move);

        } else {
            if(hoops_incor[mover] == 1) {

                // We try to optimize here by not including
                // moves that jump incorrectly.

                none_found = 1;
                for(i = 8; i >= 1; i--) {
                    if(other_side_back == 0 ||
                       (i != abs_last_move &&
                        i != 9 - abs_last_move)) {
                        if(i <= 10 - count_back_to_hoop[scores[mover]] ||
                           (10 - count_back_to_hoop[scores[mover]]) << 1 == i) {
                            none_found = 0;
                            new_move = new_move_type();
                            new_move->move = i;
                            temp_int = 10 - count_back_to_hoop[scores[mover]];
                            if(i == temp_int) {
                                link_move_into_list(moves_priority_3, new_move);
                            } else if(i == (temp_int << 1)) {
                                link_move_into_list(moves_priority_1, new_move);
                            } else if(i < temp_int) {
                                if(i == ((10 - other_side_back) << 1)) {
                                    link_move_into_list(moves_priority_1, new_move);
                                } else {
                                    link_move_into_list(moves_priority_2, new_move);
                                }
                            } else {
                                link_move_into_list(moves_priority_3, new_move);
                            }
                        }
                    }
                }
            }
            if(hoops_incor[mover] == 0 || none_found) {
                for(i = 8; i >= 1; i--) {
                    if(other_side_back == 0 ||
                       (i != abs_last_move &&
                        i != 9 - abs_last_move)) {
                        new_move = new_move_type();
                        new_move->move = i;
                        temp_int = 10 - count_back_to_hoop[scores[mover]];
                        if(i == temp_int) {
                            link_move_into_list(moves_priority_3, new_move);
                        } else if(i == (temp_int << 1)) {
                            link_move_into_list(moves_priority_1, new_move);
                        } else if(i < temp_int) {
                            if(i == ((10 - other_side_back) << 1)) {
                                link_move_into_list(moves_priority_1, new_move);
                            } else {
                                link_move_into_list(moves_priority_2, new_move);
                            }
                        } else {
                            link_move_into_list(moves_priority_3, new_move);
                        }
                    }
                }
            }
        }

        append_move_list();
        return(moves_all);
    }


void Position_type::make_move_on_board(Gen_move_type *the_move)
    // This function will make the given move on the board and update
    // any relevant variables.
    {
        if(the_move->move != 0) {
            scores[mover] += the_move->move;
            if(the_move->move > 0) {
                if(count_back_to_hoop[scores[mover]] == 0) {
                   hoop_jump_needed[mover] = the_move->move;
                } else if(count_back_to_hoop[scores[mover]] < the_move->move &&
                    count_back_to_hoop[scores[mover]] << 1 != the_move->move) {
                    hoops_incor[mover]++;
                    back_to_hoop[mover] = scores[mover] - count_back_to_hoop[scores[mover]];
                }
            } else {
                if(scores[mover] < back_to_hoop[mover]) {
                    back_to_hoop[mover] = 0;
                }
            }
            last_moves[mover] = the_move->move;
        }

        if(mover == Player1) {
            mover = Player2;
            current_move++;
        } else {
            mover = Player1;
        }
    }


int Position_type::evaluate(char comp_side)
    // This routine will evaluate the given position.
    // It will assign a value to it where the higher the number
    // the better it is for the computer.
    {
        int value = 0;
        int enemy_side;
        int comp_score;
        int enemy_score;


        glob__posits_evaled++;
        enemy_side = other_player(comp_side);
        comp_score = scores[comp_side];
        enemy_score = scores[enemy_side];

        // See if the game is over.

        if(mover == comp_side) {
            if(hoops_incor[enemy_side] >= 2) {
                return(win_value);
            }
            if(comp_side == Player1) {
                if(enemy_score > 100 && !back_to_hoop[enemy_side]) {
                    if(comp_score > 100 && !back_to_hoop[comp_side]) {
                        return(0);
                    }
                    return(-win_value);
                }
                if(comp_score > 100 && !back_to_hoop[comp_side]) {
                    return(win_value);
                }
            }
        } else {
            if(hoops_incor[comp_side] >= 2) {
                return(-win_value);
            }
            if(enemy_side == Player1) {
                if(comp_score > 100 && !back_to_hoop[comp_side]) {
                    if(enemy_score > 100 && !back_to_hoop[enemy_side]) {
                        return(0);
                    }
                    return(win_value);
                }
                if(enemy_score > 100 && !back_to_hoop[enemy_side]) {
                    return(-win_value);
                }
            }
        }

        // Quick valuation.

        value = comp_score - enemy_score;

        // Subtract 2 times the number of times we have gone over a hoop wrong.

        value -= (hoops_incor[comp_side] << 1);
        value += (hoops_incor[enemy_side] << 1);

        // If currently either side is going backward, take away points.

        if(back_to_hoop[comp_side]) {
            value -= ((comp_score - back_to_hoop[comp_side]) << 1);
        }
        if(back_to_hoop[enemy_side]) {
            value += ((enemy_score - back_to_hoop[enemy_side]) << 1);
        }

        // Remove five points for being on a hoop.  Ten if been there for more
        // than one move.

        if(!count_back_to_hoop[comp_score]) {
            if(last_moves[comp_side] == 0) {
                value -= 10;
            } else {
                value -= 5;
            }
        }
        if(!count_back_to_hoop[enemy_score]) {
            if(last_moves[enemy_side] == 0) {
                value += 10;
            } else {
                value += 5;
            }
        }

        // Return the value computed.

        return(value);
    }


void Position_type::setup_board()
    // This function will set up the board to it's starting position,
    // and it will initialize all of the other data in the class to
    // correspond to the initial position.
    {
        mover = Player1;
        scores[Player1] = 0;
        scores[Player2] = 0;
        last_moves[Player1] = 0;
        last_moves[Player2] = 0;
        hoops_incor[Player1] = 0;
        hoops_incor[Player2] = 0;
        hoop_jump_needed[Player1] = 0;
        hoop_jump_needed[Player2] = 0;
        back_to_hoop[Player1] = 0;
        back_to_hoop[Player2] = 0;
        current_move = 1;
        next = NULL;
    }


void Position_type::append_move_list()
    // This routine will put the move lists together and set moves_all
    // to the first move.  The move lists are ordered as follows:
    // moves_priority_1 to moves_priority_5.
    {
        Gen_move_type  *top;   // Record that points to top of move list.
        Gen_move_type  *temp;  // Temporary pointer.

        top = new_move_type();
        temp = top;
        temp->next = moves_priority_1;
        while(temp->next) temp = temp->next;
        temp->next = moves_priority_2;
        while(temp->next) temp = temp->next;
        temp->next = moves_priority_3;
        moves_all = top->next;
        delete_move_type(top);
    }


//
// Move free list functions.
//

static Gen_move_type *new_move_type()
    // Returns a pointer to a new Gen_move_type.
    // A free list is used to minimize time spent in malloc.
    {
        Gen_move_type *new_move;


        if(glob__move_free_list) {
            new_move = glob__move_free_list;
            glob__move_free_list = new_move->next;
        } else {
            new_move = new Gen_move_type;
        }
        new_move->val = 0;
        new_move->next_lev_moves = NULL;
        return(new_move);
    }

static Gen_move_type *new_move_type(int vl, Gen_move_type *nxt)
    // Get a new move type and initialize values.
    {
        Gen_move_type *new_move;

        new_move = new_move_type();
        new_move->val      = vl;
        new_move->next     = nxt;
        return(new_move);
    }

static void delete_move_type(Gen_move_type *move)
    // Puts the move object on free list.
    {
        move->next = glob__move_free_list;
        glob__move_free_list = move;
    }


